/* Author: seb1g13
*	Date Last Edited: 13th February 2016
*
*	Connections:
*				- Rotary Encoder PE4(Channel A), PE5(Channel B) and PE7(Push button).
*/
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "lcd.h"
#include "encoder.h"

#define SCREENWIDTH LCDHEIGHT	//Define Screen Width in landscape mode.
#define SCREENHEIGHT LCDWIDTH	//Define Screen Height in landscape mode.
#define MARGIN 10			//Allow a 10 pixel margin behind paddles.
#define PADDLEWIDTH 5			//Make paddles 5 pixels wide.
#define PADDLEHEIGHT 40			//Make paddles 40 pixels high.
#define PADDLESPEED 10			//Make paddles move 10 pixels per rotation.
#define BALLSPEED 1
#define BALLSIZE 2
#define AISPEEDFACTOR 16

void initialise();				//Initialisation function (runs once).


int main(void)
{
	initialise();
	sei(); 
	
	while(1)
	{	
		clear_screen();

		//Declare score variables.
		char s1 = '0';
		char s2= '0';
		//Define ball speed variables.
		char ballXSpeed = -1*BALLSPEED; //+ve x is moving right.
		char ballYSpeed  = BALLSPEED; //+ve y is moving down.
		//Define whether ball is "in play".
		char ballActive = 0, reset = 0;
	
		//Define opposing paddles update frequency. AI turned off for 2 player.
		uint8_t paddlePrescaler= AISPEEDFACTOR;
	
		//Define paddles at edge of screen.
		rectangle paddle1 = {MARGIN, MARGIN + PADDLEWIDTH, SCREENHEIGHT/2-PADDLEHEIGHT/2-1, SCREENHEIGHT/2+PADDLEHEIGHT/2-1};
		rectangle paddle2 = { SCREENWIDTH - (MARGIN + PADDLEWIDTH), SCREENWIDTH - MARGIN, SCREENHEIGHT/2-PADDLEHEIGHT/2-1, SCREENHEIGHT/2+PADDLEHEIGHT/2-1};
		//Define balls initial state and set ball to that state.
		rectangle initialBall = {SCREENWIDTH/2-BALLSIZE-1, SCREENWIDTH/2+BALLSIZE, SCREENHEIGHT/2-BALLSIZE-1, SCREENHEIGHT/2+BALLSIZE};
		rectangle ball = initialBall;
		//Draw elements on the screen.
		fill_rectangle(paddle1, WHITE);
		fill_rectangle(paddle2, WHITE);
		fill_rectangle(ball, WHITE);
		
		//Loop while reset is 0
		while(reset == 0)
		{
			uint8_t i = 0;
			for(i = 0; i < SCREENHEIGHT-1; i+= 10)
			{
				display.x = SCREENWIDTH/2-1;
				display.y = i;
				display_char('|');
			}
		
			//If ball is not "in play" and rotary encoder is pushed, "activate" the ball.
			if(!ballActive && !(PINE & _BV(PE7)))
			{
				ballActive = 1;
			}

			//Move paddle if within boundaries.
			
			int8_t enc_delta = os_enc_delta();
			
			fill_rectangle(paddle1, BLACK);
			paddle1.top += PADDLESPEED*enc_delta;
			paddle1.bottom += PADDLESPEED*enc_delta;
			
			if(paddle1.top < PADDLESPEED) {
				paddle1.top = PADDLESPEED;
				paddle1.bottom = PADDLESPEED+PADDLEHEIGHT;
			}
			else if(paddle1.bottom > SCREENHEIGHT-1) {
				paddle1.bottom = SCREENHEIGHT-1;
				paddle1.top = SCREENHEIGHT-1-PADDLEHEIGHT;
			}
			//Display new paddle1 position.
			fill_rectangle(paddle1, WHITE);
			
			//Move ball, if ball is active.
			if(ballActive)
			{
				fill_rectangle(ball, BLACK);
				ball.left += ballXSpeed;
				ball.right += ballXSpeed;
				ball.top -= ballYSpeed;
				ball.bottom -= ballYSpeed;
			}
			//If ball hits top or bottom boundaries, change Y direction.
			if(ball.bottom < 8 || ball.bottom > SCREENHEIGHT-8)
			{
				ballYSpeed *= -1;
			}
			//If ball hits paddle1 or 2, change X direction.
			if((ball.left < 18 && ball.bottom > paddle1.top && ball.top < paddle1.bottom) || (ball.right > SCREENWIDTH -19 && ball.bottom > paddle2.top && ball.top < paddle2.bottom))
			{
				ballXSpeed *= -1;
			}
			//If ball misses paddle1, add to player 2's score and reset ball.
			if(ball.left < MARGIN)
			{
				s2++;
				ball = initialBall;
				ballXSpeed *= -1;
				if(s2 > '9')
				{
					ballActive = 0;
					display.x = SCREENWIDTH/2 - 35;
					display.y = SCREENHEIGHT/2 - 9;
					display_string("Player 2 Wins!");
					while((PINE & _BV(PE7)));
					display.x = SCREENWIDTH/2 - 35;
					display.y = SCREENHEIGHT/2 - 9;
					display_string("              ");
					reset = 1;
				}
			}
			//If ball misses paddle2, add to player 1's score and reset ball.
			if(ball.right > SCREENWIDTH - MARGIN)
			{
				s1++;
				ball = initialBall;
				ballXSpeed *= -1;
				if(s1 > '9')
				{
					ballActive = 0;
					display.x = SCREENWIDTH/2 - 35;
					display.y = SCREENHEIGHT/2 - 9;
					display_string("Player 1 Wins!");
					while((PINE & _BV(PE7)));// && (PINC & _BV(PC3)));
					display.x = SCREENWIDTH/2 - 35;
					display.y = SCREENHEIGHT/2 - 9;
					display_string("              ");
					reset = 1;
				}
			}
			//If ball is active, display ball, update opposing paddle and player scores.
			if(ballActive)
			{
				fill_rectangle(ball, WHITE);
				display.y = 20;
				display.x = SCREENWIDTH/4 - 6;
				display_char(s1);
				display.x = 3*SCREENWIDTH/4 - 6;
				display_char(s2);
				_delay_ms(5);
			
				
				paddlePrescaler--;
				if(paddlePrescaler < 1)
				{
					fill_rectangle(paddle2, BLACK);
					if((paddle2.top + PADDLEHEIGHT/2) < ball.bottom)
						paddle2.top+=PADDLESPEED, paddle2.bottom+=PADDLESPEED;
					if((paddle2.top + PADDLEHEIGHT/2) > ball.top)
						paddle2.top-=PADDLESPEED, paddle2.bottom-=PADDLESPEED;
					if(paddle2.top < PADDLESPEED) {
						paddle2.top = PADDLESPEED;
						paddle2.bottom = PADDLESPEED+PADDLEHEIGHT;
					}
					else if(paddle2.bottom > SCREENHEIGHT-1) {
						paddle2.bottom = SCREENHEIGHT-1;
						paddle2.top = SCREENHEIGHT-1-PADDLEHEIGHT;
					}
					fill_rectangle(paddle2, WHITE);
					paddlePrescaler=AISPEEDFACTOR;
				}
				
			}
		}
	}
	return 1;
}

void initialise()
{
	/* 8MHz clock, no prescaling (DS, p. 48) */
	CLKPR = (1 << CLKPCE);
	CLKPR = 0;
	
	init_encoder();
	
	/* Timer 0 for switch scan interrupt: */

	TCCR0A = _BV(WGM01);
	TCCR0B = _BV(CS01)
          | _BV(CS00);   /* F_CPU / 64 */
          

    /* SET OCR0A FOR A 1 MS PERIOD */   
	
	OCR0A |= 61; /* OCRnx = (f_clk_IO / (2*N*f_OCnx)) - 1
					f_clk_IO = 8,000,000
					N = 64
					f_OCnx = 1000
					OCRnx = 61
				*/
    		
    /* ENABLE TIMER INTERRUPT */
	
	TIMSK0 = _BV(OCIE0A); /* Set Timer0 Output Campare Match A Interrupt Enable bit
							 in Timer0 Interrupt Mask Register */
							 
	//Initialise LCD with West orientation.
	init_lcd();
	set_orientation(West);
}

ISR( TIMER0_COMPA_vect ) {
    
	cli();
     scan_encoder();
	sei();
}